/*
 *   Copyright (c) Texas Instruments Incorporated 2016-2019
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

/**
 *  \file     app_mcan2.c
 *
 *  \brief    This file contains MCAN sample code.
 *
 *  \details  MCAN operational mode is set to Classic CAN.
 *
 */

/* ========================================================================== */
/*                             Include Files                                  */
/* ========================================================================== */
#define TIMER_CALL
#include <stdint.h>
#include <stdio.h>
#include <ti/csl/csl_types.h>
#include <ti/csl/soc.h>
#include <ti/csl/hw_types.h>
#include <ti/csl/arch/csl_arch.h>
#include <ti/csl/csl_mcan.h>
#include <ti/drv/uart/UART.h>
#include <ti/drv/uart/UART_stdio.h>
#include <ti/osal/osal.h>
#include <ti/drv/gpio/GPIO.h>
#include <ti/drv/gpio/soc/GPIO_soc.h>
#include <ti/csl/csl_gpio.h>
#include <ti/board/board.h>
#include <ti/drv/i2c/I2C.h>
#if defined (SOC_J721E)
#include <ti/drv/sciclient/sciclient.h>
#endif
#include <utils/mcan/app_mcan2.h>
#include <utils/remote_service/include/app_remote_service.h>
#include <utils/console_io/include/app_log.h>

#include <ti/drv/ipc/ipc.h>
#include <ti/osal/SemaphoreP.h>
#include "app_CAN_reader.h"
#include "app_sciclient_helper.h"
#include <ti/sysbios/knl/Task.h>

#include <ti/board/src/j721e_evm/include/board_cfg.h>
#include <ti/board/src/j721e_evm/include/board_pinmux.h>
#include <ti/csl/soc/cslr_soc_ctrl_mmr.h>
#ifdef TIMER_CALL
#include <ti/sysbios/timers/dmtimer/Timer.h>
#include <ti/osal/TimerP.h>
#include <ti/drv/uart/UART_stdio.h>
#include <ti/drv/uart/UART.h>
static uintptr_t SM_addr = NULL;
#endif
static uint32_t FLAG_SM = 0;

static uint8_t gTskTxStackMain[8*1024]
__attribute__ ((section(".bss:taskStackSection")))
__attribute__ ((aligned(8192)))
    ;
static uint8_t gTskRxStackMain[8*1024]
__attribute__ ((section(".bss:taskStackSection")))
__attribute__ ((aligned(8192)));

/* ========================================================================== */
/*                                Macros                                      */
/* ========================================================================== */
#if defined (SOC_AM65XX)
#define APP_ENABLE_UART_PRINT                    (1U)
#elif defined (SOC_J721E)
#define APP_ENABLE_UART_PRINT                    (1U)
#endif

#define APP_MCAN_STD_ID_FILT_START_ADDR          (0U)
#define APP_MCAN_STD_ID_FILTER_NUM               (1U)
#define APP_MCAN_EXT_ID_FILT_START_ADDR          (48U)
#define APP_MCAN_EXT_ID_FILTER_NUM               (1U)
#define APP_MCAN_TX_EVENT_START_ADDR             (100U)
#define APP_MCAN_TX_EVENT_SIZE                   (5U)
#define APP_MCAN_TX_BUFF_START_ADDR              (148U)
#define APP_MCAN_TX_BUFF_SIZE                    (5U)
#define APP_MCAN_TX_FIFO_SIZE                    (5U)
#define APP_MCAN_FIFO_0_START_ADDR               (548U)
#define APP_MCAN_FIFO_0_NUM                      (5U)
#define APP_MCAN_FIFO_1_START_ADDR               (748U)
#define APP_MCAN_FIFO_1_NUM                      (5U)
#define APP_MCAN_RX_BUFF_START_ADDR              (948U)

#define APP_MCAN_EXT_ID_AND_MASK                 (0x1FFFFFFFU)

#define APP_MCU_MCAN_0_INT0                                   (0U)
#define APP_MCU_MCAN_0_INT1                                   (1U)
#define APP_MCU_MCAN_0_TS_INT                                 (4U)
#define APP_MCU_MCAN_1_INT0                                   (2U)
#define APP_MCU_MCAN_1_INT1                                   (3U)
#define APP_MCU_MCAN_1_TS_INT                                 (5U)

#define I2C_BUS_NUM                     (0)
#define TCA6424_SLAVE_ADDR              ((uint8_t) 0x23U)

#define TCA6424_CMD_AUTO_INC            ((uint8_t) 0x80U)

#if defined (SOC_J721E)
#if defined (BUILD_MCU1_0)
#define APP_MAIN_MCAN_2_INT0                (CSLR_MCU_R5FSS0_CORE0_INTR_MAIN2MCU_LVL_INTRTR0_OUTL_0)
#define APP_MAIN_MCAN_2_INT1                (CSLR_MCU_R5FSS0_CORE0_INTR_MAIN2MCU_LVL_INTRTR0_OUTL_1)
#define APP_MAIN_MCAN_2_TS_INT              (CSLR_MCU_R5FSS0_CORE0_INTR_MAIN2MCU_LVL_INTRTR0_OUTL_2)
#else
#define APP_MAIN_MCAN_0_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN0_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_0_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN0_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_0_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN0_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_2_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN2_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_2_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN2_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_2_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN2_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_4_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN4_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_4_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN4_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_4_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN4_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_5_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN5_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_5_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN5_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_5_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN5_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_6_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN6_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_6_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN6_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_6_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN6_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_7_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN7_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_7_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN7_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_7_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN7_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_9_INT0                (CSLR_R5FSS1_CORE0_INTR_MCAN9_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_9_INT1                (CSLR_R5FSS1_CORE0_INTR_MCAN9_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_9_TS_INT              (CSLR_R5FSS1_CORE0_INTR_MCAN9_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#define APP_MAIN_MCAN_11_INT0               (CSLR_R5FSS1_CORE0_INTR_MCAN11_MCANSS_MCAN_LVL_INT_0)
#define APP_MAIN_MCAN_11_INT1               (CSLR_R5FSS1_CORE0_INTR_MCAN11_MCANSS_MCAN_LVL_INT_1)
#define APP_MAIN_MCAN_11_TS_INT             (CSLR_R5FSS1_CORE0_INTR_MCAN11_MCANSS_EXT_TS_ROLLOVER_LVL_INT_0)
#endif
#endif

/* Print buffer character limit for prints- UART or CCS Console */
#define APP_PRINT_BUFFER_SIZE                       (4000)
#if defined (SOC_AM65XX)
/* Port and pin number mask for GPIO Load pin.
   Bits 7-0: Pin number  and Bits 15-8: Port number */
#define AM65XX_IDK_GPIO_CAN0_STB_PIN    (0x012F)
#define AM65XX_IDK_GPIO_CAN1_STB_PIN    (0x0143)
#endif
#if defined (SOC_J721E)
#define J721E_GPIO_GESI_CAN_STB_PIN     (0x003C)
#define J721E_GPIO_MAIN_MCAN2_STB_PIN   (0x007F)
#endif
/*****************************/
/*****************************/
#define TX_CODE 0
#define RX_CODE 0
#define TX_RX_CODE 1

volatile uint32_t gMcanIsrIntr1Status;
volatile uint32_t num_Tx=0,num_Rx=0,num_ISR=0;
/*****************************/
/*****************************/
/* ========================================================================== */
/*                            Global Variables                                */
/* ========================================================================== */
#define APP_USE_CAN
#if defined (APP_USE_CAN)
static pinmuxPerCfg_t gMcan2PinCfg[] =
{
    /* MyMCAN2 -> MCAN2_RX -> AC2 */
    {
        PIN_UART0_CTSN, PIN_MODE(3) | \
        ((PIN_PULL_DISABLE | PIN_INPUT_ENABLE) & (~PIN_PULL_DIRECTION))
    },
    /* MyMCAN2 -> MCAN2_TX -> AB1 */
    {
        PIN_UART0_RTSN, PIN_MODE(3) | \
        ((PIN_PULL_DISABLE) & (~PIN_PULL_DIRECTION & ~PIN_INPUT_ENABLE))
    },
    {PINMUX_END}
};

static pinmuxPerCfg_t gI2c0PinCfg[] =
{
    /* MyI2C0 -> I2C0_SCL -> AC5 */
    {
        PIN_I2C0_SCL, PIN_MODE(0) | \
        ((PIN_PULL_DIRECTION | PIN_INPUT_ENABLE) & (~PIN_PULL_DISABLE))
    },
    /* MyI2C0 -> I2C0_SDA -> AA5 */
    {
        PIN_I2C0_SDA, PIN_MODE(0) | \
        ((PIN_PULL_DIRECTION | PIN_INPUT_ENABLE) & (~PIN_PULL_DISABLE))
    },
    {PINMUX_END}
};

static pinmuxPerCfg_t gGpio0PinCfg[] =
{
    /* MyGPIO0 -> GPIO0_127 -> AC4 */
    {
        PIN_UART1_CTSN, PIN_MODE(7) | \
        ((PIN_PULL_DISABLE | PIN_INPUT_ENABLE) & (~PIN_PULL_DIRECTION))
    },
    {PINMUX_END}
};

static pinmuxModuleCfg_t gMcanPinCfg[] =
{
    {0, TRUE, gMcan2PinCfg},
    {1, TRUE, gI2c0PinCfg},
    {2, TRUE, gGpio0PinCfg},
    {PINMUX_END}
};
#endif

static pinmuxBoardCfg_t gCockpitDemoPinmuxDataInfo[] =
{
#if defined (APP_USE_CAN)
    {1, gMcanPinCfg},
#endif
    {PINMUX_END}
};

uint32_t          gMcanAppdataSize[16] =
{0, 1, 2, 3, 4, 5, 6, 7, 8, 12, 16, 20, 24, 32, 48, 64};
volatile uint32_t gMcanIsrIntr0Flag = 1U;
//volatile uint32_t gMcanIsrIntr1Flag = 1U;
SemaphoreP_Handle RxgSyncSem;
SemaphoreP_Handle TxgSyncSem;
MCAN_ECCErrStatus gMcaneccErr;
#define APP_MCAN2_BASEADDR                       (CSL_MCAN2_MSGMEM_RAM_BASE)
uint32_t gMcanModAddr = APP_MCAN2_BASEADDR;

MCAN_TxBufElement          txMsg;
MCAN_RxBufElement    rxMsg;
MCAN_MsgRAMConfigParams    msgRAMConfigParams;
MCAN_ConfigParams          configParams;

const uint32_t mcan2TxMsgId[] = {OBJ_MSG_ID1,OBJ_MSG_ID2,OBJ_MSG_ID3,OBJ_MSG_ID4,OBJ_MSG_ID5,OBJ_MSG_ID6,OBJ_MSG_ID7,OBJ_MSG_ID8,OBJ_MSG_ID9,OBJ_MSG_ID10,OBJ_MSG_ID11,OBJ_MSG_ID12,OBJ_MSG_ID13,OBJ_MSG_ID14,OBJ_MSG_ID15,OBJ_MSG_ID16,OBJ_MSG_ID17,OBJ_MSG_ID18,OBJ_MSG_ID19,OBJ_MSG_ID20,\
                                 OBJ_MSG_ID21,OBJ_MSG_ID22,OBJ_MSG_ID23,OBJ_MSG_ID24,OBJ_MSG_ID25,OBJ_MSG_ID26,OBJ_MSG_ID27,OBJ_MSG_ID28,OBJ_MSG_ID29,OBJ_MSG_ID30,OBJ_MSG_ID31,OBJ_MSG_ID32,OBJ_MSG_ID33,OBJ_MSG_ID34,OBJ_MSG_ID35,OBJ_MSG_ID36,OBJ_MSG_ID37,OBJ_MSG_ID38,OBJ_MSG_ID39,\
                                 OBJ_MSG_ID40,OBJ_MSG_ID41
};



#if defined (SOC_J721E)
/* GPIO Driver board specific pin configuration structure */
GPIO_PinConfig gpioPinConfigs[] = {
    /* Output pin : CAN STB */
    J721E_GPIO_GESI_CAN_STB_PIN   | GPIO_CFG_OUTPUT,
    J721E_GPIO_MAIN_MCAN2_STB_PIN | GPIO_CFG_OUTPUT,
};
#endif

/* GPIO Driver call back functions */
GPIO_CallbackFxn gpioCallbackFunctions[] = {
    NULL
};

/* GPIO Driver configuration structure */
GPIO_v0_Config GPIO_v0_config = {
        gpioPinConfigs,
        gpioCallbackFunctions,
        sizeof(gpioPinConfigs) / sizeof(GPIO_PinConfig),
        sizeof(gpioCallbackFunctions) / sizeof(GPIO_CallbackFxn),
        0,
    };
/* ========================================================================== */
/*                 Internal Function Declarations                             */
/* ========================================================================== */

/**
 * \brief   This function will configure MCAN module
 *
 * \param   none.
 *
 * \retval  status      configuration status.
 */
static int32_t App_mcanConfig(void);

int32_t appRemoteServiceMCAN2Handler(char *service_name, uint32_t cmd,
    void *prm, uint32_t prm_size, uint32_t flags);
/**
 * \brief   This function will configure X-BAR for MCAN interrupts
 *
 * \param   none.
 *
 * \retval  status      configuration status.
 */
static int32_t App_mcanXBarConfig(void);

/**
 * \brief   This is Interrupt Service Routine for MCAN interrupt 0.
 *
 * \param   none.
 *
 * \retval  none.
 */
static void App_mcanIntr01ISR(uintptr_t arg);
/**
 * \brief   This API will load the register from ECC memory bank.
 *
 * \param   txMsg           message object to be transmitted.
 *
 * \return  None.
 */
static int32_t App_mcanRegisterInterrupt(uint32_t intNum, void f(uintptr_t));

/* ========================================================================== */
/*                          Function Definitions                              */
/* ========================================================================== */
#define TCA6424_CMD_AUTO_INC            ((uint8_t) 0x80U)

/* Input status register */
#define TCA6424_REG_INPUT0              ((UInt8) 0x00U)
#define TCA6424_REG_INPUT1              ((UInt8) 0x01U)
#define TCA6424_REG_INPUT2              ((UInt8) 0x02U)

/* Output register to change state of output BIT set to 1, output set HIGH */
#define TCA6424_REG_OUTPUT0             ((uint8_t) 0x04U)
#define TCA6424_REG_OUTPUT1             ((uint8_t) 0x05U)
#define TCA6424_REG_OUTPUT2             ((uint8_t) 0x06U)

/* Configuration register. BIT = '1' sets port to input, BIT = '0' sets
 * port to output */
#define TCA6424_REG_CONFIG0             ((uint8_t) 0x0CU)
#define TCA6424_REG_CONFIG1             ((uint8_t) 0x0DU)
#define TCA6424_REG_CONFIG2             ((uint8_t) 0x0EU)



void SetupI2CTransfer(I2C_Handle handle,  uint32_t slaveAddr,
                      uint8_t *writeData, uint32_t numWriteBytes,
                      uint8_t *readData,  uint32_t numReadBytes)
{
    bool status;
    I2C_Transaction i2cTransaction;

    I2C_transactionInit(&i2cTransaction);
    i2cTransaction.slaveAddress = slaveAddr;
    i2cTransaction.writeBuf = (uint8_t *)&writeData[0];
    i2cTransaction.writeCount = numWriteBytes;
    i2cTransaction.readBuf = (uint8_t *)&readData[0];
    i2cTransaction.readCount = numReadBytes;
    status = I2C_transfer(handle, &i2cTransaction);
    if(FALSE == status)
    {
        UART_printf("\n Data Transfer failed. \n");
    }
}

void padConfig_prcmEnable()
{
    /* UART Init */
    Board_initCfg   boardCfg;
    Board_STATUS    boardStatus;

#if defined (SOC_J721E)
    I2C_Params      i2cParams;
    I2C_Handle      handle = NULL;
    uint8_t         dataToSlave[4];
#endif

    boardCfg = BOARD_INIT_MODULE_CLOCK |
               BOARD_INIT_PINMUX_CONFIG;
#if (APP_ENABLE_UART_PRINT == 1)
    boardCfg |= BOARD_INIT_UART_STDIO;
#endif
    boardStatus = Board_init(boardCfg);
    if (boardStatus != BOARD_SOK)
    {
        UART_printf("[Error] Board init failed!!\n");
    }

    /* Pin mux for CAN STB used in GESI board */
    *(volatile unsigned int *)(0x0011c0f4) = 0x20007;
    /* Pinmux for MAIN_MCAN4 */
    *(volatile unsigned int *)(0x0011c020) = 0x60006;
    *(volatile unsigned int *)(0x0011c024) = 0x60006;
    /* Pinmux for MAIN_MCAN5 */
    *(volatile unsigned int *)(0x0011c04c) = 0x60006;
    *(volatile unsigned int *)(0x0011c050) = 0x60006;
    /* Pinmux for MAIN_MCAN6 */
    *(volatile unsigned int *)(0x0011c054) = 0x60006;
    *(volatile unsigned int *)(0x0011c06C) = 0x60006;
    /* Pinmux for MAIN_MCAN7 */
    *(volatile unsigned int *)(0x0011c074) = 0x60006;
    *(volatile unsigned int *)(0x0011c078) = 0x60006;
    /* Pinmux for MAIN_MCAN9 */
    *(volatile unsigned int *)(0x0011c0cc) = 0x60006;
    *(volatile unsigned int *)(0x0011c0d0) = 0x60006;
    /* Pinmux for MAIN_MCAN11 */
    *(volatile unsigned int *)(0x0011c11c) = 0x60006;
    *(volatile unsigned int *)(0x0011c120) = 0x60006;

    /* GPIO initialization */
    GPIO_init();

    /* Enable CAN transceivers by setting the STB pins */
#if defined (SOC_AM65XX)
    GPIO_write(0, GPIO_PIN_HIGH);
    GPIO_write(1, GPIO_PIN_HIGH);
#elif defined (SOC_J721E)
    /* Enable the TCAN on GESI board.
     * Main Domain MCAN instances 4,5,6,7,9,11.
     */
    GPIO_write(0, GPIO_PIN_LOW);

    /* Enable MCU MCAN 1, WKUP_GPIO0_2. */
    GPIOSetDirMode_v0(CSL_WKUP_GPIO0_BASE, 2, GPIO_DIRECTION_OUTPUT);
    GPIOPinWrite_v0(CSL_WKUP_GPIO0_BASE, 2, GPIO_PIN_LOW);

    /* Enable MCU MCAN 0, WKUP_GPIO0_0, WKUP_GPIO0_54 */
    GPIOSetDirMode_v0(CSL_WKUP_GPIO0_BASE, 0, GPIO_DIRECTION_OUTPUT);
    GPIOPinWrite_v0(CSL_WKUP_GPIO0_BASE, 0, GPIO_PIN_HIGH);
    GPIOSetDirMode_v0(CSL_WKUP_GPIO0_BASE, 54, GPIO_DIRECTION_OUTPUT);
    GPIOPinWrite_v0(CSL_WKUP_GPIO0_BASE, 54, GPIO_PIN_HIGH);

    /* Enable Main MCAN 2, GPIO0_127. */
    GPIO_write(1, GPIO_PIN_LOW);

    /*
     * Configuring TCA6424 IO Exp 2 with addr 0x22
     * This io expander is controlled by i2c0
     * For Main MCAN2 P13 and P14 should be set to 0, This should route the MCAN2 STB line to transciver.
     * For Main MCAN0 P06 and P07 should be set to 1.
     */
    /* I2C initialization */
    I2C_init();
    I2C_Params_init(&i2cParams);
    i2cParams.transferMode = I2C_MODE_BLOCKING;
    i2cParams.bitRate = I2C_400kHz;
    i2cParams.transferCallbackFxn = NULL;

    handle = I2C_open(0U, &i2cParams);
    printf("I2C_open .........\n");

    dataToSlave[0] = TCA6424_REG_CONFIG0 | TCA6424_CMD_AUTO_INC;
    dataToSlave[1] = 0x0U;
    SetupI2CTransfer(handle, 0x23, &dataToSlave[0], 2, NULL, 0);
    printf("SetupI2CTransfer .........\n");

    dataToSlave[0] = TCA6424_REG_INPUT0 | TCA6424_CMD_AUTO_INC;
    dataToSlave[1] = 0x0U;
    dataToSlave[2] = 0x0U;
    dataToSlave[3] = 0x0U;
    SetupI2CTransfer(handle, 0x23, &dataToSlave[0], 1, &dataToSlave[1], 3);

    /* Set P06 and P07 to 1.
     * Set P13 and P14 to 0.
     */
    dataToSlave[0] = TCA6424_REG_OUTPUT0 | TCA6424_CMD_AUTO_INC;
    dataToSlave[1] |= 0xC0;
    dataToSlave[2] &= ~(0x18);
    SetupI2CTransfer(handle, 0x23, &dataToSlave[0], 1, &dataToSlave[1], 3);
#endif
}
#if defined(R5F) && defined(SYSBIOS)
int32_t appMcan2Init(void)
{
    int32_t                    configStatus = CSL_PASS;
    uint32_t                  /* mode         = 0U, */loopCnt = 0U;

    /* Do Pad Config for UART and MCAN */
    padConfig_prcmEnable();

    printf("\n START MCAN2 INIT....\n");

    /* CrossBar Configuration */
    configStatus = App_mcanXBarConfig();

    /* Initialize message to transmit */
    txMsg.id  = (uint32_t)((uint32_t)(0x4U) << 18U);
    txMsg.rtr = 0U;
    txMsg.xtd = 0U;
    txMsg.esi = 0U;
    txMsg.dlc = 0xFU;
    txMsg.brs = 1U;
    txMsg.fdf = 0u;//1U;
    txMsg.efc = 1U;
    txMsg.mm  = 0xAAU;
    for (loopCnt = 0; loopCnt < MCAN_MAX_PAYLOAD_BYTES; loopCnt++)
    {
        txMsg.data[loopCnt] = 0;
    }
    configStatus = App_mcanConfig();

    Rx_Timer_task();
    Tx_Timer_task();

    printf("\n MCAN2 INIT END..........\n");

    return 0;
}
#endif

/* ========================================================================== */
/*                 Internal Function Definitions                              */
/* ========================================================================== */
static int32_t App_mcanRegisterInterrupt(uint32_t intNum, void f(uintptr_t))
{
    int32_t configStatus = STW_SOK;
    OsalRegisterIntrParams_t    intrPrms;
    OsalInterruptRetCode_e      osalRetVal;
    HwiP_Handle                 hwiHandle = NULL;

    /* Enable CPU Interrupts and register ISR - MCAN0 Intr0 */
    Osal_RegisterInterrupt_initParams(&intrPrms);
    /* Populate the interrupt parameters */
    intrPrms.corepacConfig.arg              = (uintptr_t) NULL;
    intrPrms.corepacConfig.isrRoutine       = f;
    intrPrms.corepacConfig.priority         = 0U;
    intrPrms.corepacConfig.corepacEventNum  = 0U;
    intrPrms.corepacConfig.intVecNum        = intNum;

    /* Register interrupts */
    osalRetVal = Osal_RegisterInterrupt(&intrPrms, &hwiHandle);
    if(OSAL_INT_SUCCESS != osalRetVal)
    {
        configStatus = CSL_EFAIL;
    }
    return configStatus;
}

static int32_t App_mcanXBarConfig(void)
{
    int32_t configStatus = STW_SOK;
    /* Main MCAN Inst 2 */
#if TX_CODE
    configStatus =  App_mcanRegisterInterrupt(APP_MAIN_MCAN_2_INT0, &App_mcanIntr0ISR);
#elif RX_CODE
    configStatus = App_mcanRegisterInterrupt (APP_MAIN_MCAN_2_INT1, &App_mcanIntr1ISR);
#else
    configStatus = App_mcanRegisterInterrupt (APP_MAIN_MCAN_2_INT0, &App_mcanIntr01ISR);
#endif
    if(STW_SOK != configStatus)
    {
        UART_printf("CrossBar/Interrupt Configuration failed.\n");
    }
    else
    {
        UART_printf("CrossBar/Interrupt Configuration done.\n");
    }

    return configStatus;
}

static int32_t App_mcanConfig(void)
{
    uint32_t                   fdoe;
    int32_t                    configStatus = CSL_PASS;
    MCAN_RevisionId            revId;
    MCAN_InitParams            initParams;
    MCAN_StdMsgIDFilterElement stdFiltelem;
    MCAN_BitTimingParams       bitTimes;

    /* Initialize MCAN Init params */
#ifdef MCAN_500KBPS
    initParams.fdMode          = 0x0U;
#else
    initParams.fdMode          = 0x1U;
#endif
    initParams.brsEnable       = 0x1U;
    initParams.txpEnable       = 0x0U;
    initParams.efbi            = 0x0U;
    initParams.pxhddisable     = 0x0U;
    initParams.darEnable       = 0x0u;//0x1U;
    initParams.wkupReqEnable   = 0x1U;
    initParams.autoWkupEnable  = 0x1U;
    initParams.emulationEnable = 0x1U;
    initParams.emulationFAck   = 0x0U;
    initParams.clkStopFAck     = 0x0U;
    initParams.wdcPreload      = 0xFFU;
    initParams.tdcEnable       = 0x1U;
    initParams.tdcConfig.tdcf  = 0xAU;
    initParams.tdcConfig.tdco  = 0x6U;
    /* Initialize MCAN Config params */
    configParams.monEnable         = 0x0U;
    configParams.asmEnable         = 0x0U;
    configParams.tsPrescalar       = 0xFU;
    configParams.tsSelect          = 0x0U;
    configParams.timeoutSelect     = MCAN_TIMEOUT_SELECT_CONT;
    configParams.timeoutPreload    = 0xFFFFU;
    configParams.timeoutCntEnable  = 0x0U;
    configParams.filterConfig.rrfs = 0x1U;
    configParams.filterConfig.rrfe = 0x1U;
    configParams.filterConfig.anfe = 0x1U;
    configParams.filterConfig.anfs = 0x1U;
    /* Initialize Message RAM Sections Configuration Parameters */
    msgRAMConfigParams.flssa                = APP_MCAN_STD_ID_FILT_START_ADDR;
    msgRAMConfigParams.lss                  = APP_MCAN_STD_ID_FILTER_NUM;
    msgRAMConfigParams.flesa                = APP_MCAN_EXT_ID_FILT_START_ADDR;
    msgRAMConfigParams.lse                  = APP_MCAN_EXT_ID_FILTER_NUM;
    msgRAMConfigParams.txStartAddr          = APP_MCAN_TX_BUFF_START_ADDR;
    msgRAMConfigParams.txBufNum             = APP_MCAN_TX_BUFF_SIZE;//32
    msgRAMConfigParams.txFIFOSize           = 0U;
    msgRAMConfigParams.txBufMode            = 0U;
    msgRAMConfigParams.txBufElemSize        = MCAN_ELEM_SIZE_64BYTES;//8
    msgRAMConfigParams.txEventFIFOStartAddr = APP_MCAN_TX_EVENT_START_ADDR;
    msgRAMConfigParams.txEventFIFOSize      = 10u;//APP_MCAN_TX_BUFF_SIZE;
    msgRAMConfigParams.txEventFIFOWaterMark = 3U;
    msgRAMConfigParams.rxFIFO0startAddr     = APP_MCAN_FIFO_0_START_ADDR;
    msgRAMConfigParams.rxFIFO0size          = APP_MCAN_FIFO_0_NUM;
    msgRAMConfigParams.rxFIFO0waterMark     = 3U;
    msgRAMConfigParams.rxFIFO0OpMode        = 0U;
    msgRAMConfigParams.rxFIFO1startAddr     = APP_MCAN_FIFO_1_START_ADDR;
    msgRAMConfigParams.rxFIFO1size          = 10;//APP_MCAN_FIFO_1_NUM;
    msgRAMConfigParams.rxFIFO1waterMark     = 3U;
    msgRAMConfigParams.rxFIFO1OpMode        = 0U;
    msgRAMConfigParams.rxBufStartAddr       = APP_MCAN_RX_BUFF_START_ADDR;
    msgRAMConfigParams.rxBufElemSize        = MCAN_ELEM_SIZE_64BYTES;
    msgRAMConfigParams.rxFIFO0ElemSize      = MCAN_ELEM_SIZE_64BYTES;
    msgRAMConfigParams.rxFIFO1ElemSize      = MCAN_ELEM_SIZE_64BYTES;
    /* Initialize Tx Buffer Config params */
    stdFiltelem.sfid2 = 0x0U;
    stdFiltelem.sfid1 = 0x0U;;//0x4U;
    stdFiltelem.sfec  = 0x0U;;//0x7U;
    stdFiltelem.sft   = 0x0U;
    /* Initialize bit timings
     * Configuring 1Mbps and 5Mbps as nominal and data bit-rate respectively */
#ifdef MCAN_500KBPS
    bitTimes.nomRatePrescalar   = 0xFU;
#else
    bitTimes.nomRatePrescalar   = 0x7U;
#endif
    bitTimes.nomTimeSeg1        = 0x5U;
    bitTimes.nomTimeSeg2        = 0x2U;
    bitTimes.nomSynchJumpWidth  = 0x0U;
    bitTimes.dataRatePrescalar  = 0x1U;
    bitTimes.dataTimeSeg1       = 0x3U;
    bitTimes.dataTimeSeg2       = 0x2U;
    bitTimes.dataSynchJumpWidth = 0x0U;

    /* Get MCANSS Revision ID */
    MCAN_getRevisionId(gMcanModAddr, &revId);
    UART_printf("MCANSS Revision ID:\n");
    UART_printf("scheme:0x%x\n", revId.scheme);
    UART_printf("Business Unit:0x%x\n", revId.bu);
    UART_printf("Module ID:0x%x\n", revId.modId);
    UART_printf("RTL Revision:0x%x\n", revId.rtlRev);
    UART_printf("Major Revision:0x%x\n", revId.major);
    UART_printf("Custom Revision:0x%x\n", revId.custom);
    UART_printf("Minor Revision:0x%x\n", revId.minor);
    /* Enable Auto wakeup */
    fdoe = MCAN_isFDOpEnable(gMcanModAddr);
    if ((uint32_t)TRUE == fdoe)
    {
        UART_printf("CAN-FD operation is enabled through E-Fuse.\n");
    }
    else
    {
        UART_printf("CAN-FD operation is disabled through E-Fuse.\n");
    }
    /* wait for memory initialization to happen */
    while (FALSE == MCAN_isMemInitDone(gMcanModAddr))
    {}
    /* Get endianess value */
    UART_printf("Endianess Value: 0x%x\n",
                       MCAN_getEndianVal(gMcanModAddr));
    /* Put MCAN in SW initialization mode */
    MCAN_setOpMode(gMcanModAddr, MCAN_OPERATION_MODE_SW_INIT);
    while (MCAN_OPERATION_MODE_SW_INIT != MCAN_getOpMode(gMcanModAddr))
    {}
    /* Initialize MCAN module */
    MCAN_init(gMcanModAddr, &initParams);
    /* Configure MCAN module */
    MCAN_config(gMcanModAddr, &configParams);
    /* Configure Bit timings */
    MCAN_setBitTime(gMcanModAddr, &bitTimes);
    /* Set Extended ID Mask */
    MCAN_setExtIDAndMask(gMcanModAddr, APP_MCAN_EXT_ID_AND_MASK);
    /* Configure Message RAM Sections */
    MCAN_msgRAMConfig(gMcanModAddr, &msgRAMConfigParams);
    /* Configure Standard ID filter element */
    MCAN_addStdMsgIDFilter(gMcanModAddr, 0U, &stdFiltelem);
    /* Take MCAN out of the SW initialization mode */
    MCAN_setOpMode(gMcanModAddr, MCAN_OPERATION_MODE_NORMAL);
    while (MCAN_OPERATION_MODE_NORMAL != MCAN_getOpMode(gMcanModAddr))
    {}

    /* Enable Interrupts */
    MCAN_enableIntr(gMcanModAddr, MCAN_INTR_MASK_ALL, (uint32_t)TRUE);
    MCAN_enableIntr(gMcanModAddr,
                    MCAN_INTR_SRC_RES_ADDR_ACCESS, (uint32_t)FALSE);

    /* Select Interrupt Line */
    MCAN_selectIntrLine(APP_MCAN2_BASEADDR,
                        MCAN_INTR_MASK_ALL,
                        MCAN_INTR_LINE_NUM_0);

    /* Enable Interrupt Line */
    MCAN_enableIntrLine(APP_MCAN2_BASEADDR,
                        MCAN_INTR_LINE_NUM_0,
                        1U);
    return configStatus;
}

static void App_mcanIntr01ISR(uintptr_t arg)
{
    uint32_t isrType = 2; //set 0 in case of Tx, 1 in case of Rx and 2 is when error interrupt
    gMcanIsrIntr1Status = MCAN_getIntrStatus(gMcanModAddr);
    MCAN_clearIntrStatus(gMcanModAddr, gMcanIsrIntr1Status);
    if (MCAN_INTR_SRC_TRANS_COMPLETE ==
        (gMcanIsrIntr1Status & MCAN_INTR_SRC_TRANS_COMPLETE))
    {
        isrType = 0;
        SemaphoreP_post(TxgSyncSem);
        num_ISR++;
    }

    if(((gMcanIsrIntr1Status & MCAN_INTR_SRC_RX_FIFO0_NEW_MSG) ==
         MCAN_INTR_SRC_RX_FIFO0_NEW_MSG) ||
       ((gMcanIsrIntr1Status & MCAN_INTR_SRC_RX_FIFO1_NEW_MSG) ==
         MCAN_INTR_SRC_RX_FIFO1_NEW_MSG))
    {
        isrType = 1;
        SemaphoreP_post(RxgSyncSem);
        num_ISR++;
    }
    if(isrType == 2)
    {
        UART_printf("ERROR ISR, Interrupt Status: 0x%x!!\n", gMcanIsrIntr1Status);
    }
}

int32_t appRemoteServiceMcan2Init()
{
    int32_t status = 0;

    status = appRemoteServiceRegister(
        APP_REMOTE_SERVICE_MCAN2_NAME, appRemoteServiceMCAN2Handler);
    if(status!=0)
    {
        UART_printf(
            " REMOTE_SERVICE_MCAN2: ERROR: Unable to register remote service MCAN2 handler\n");
    }
    else
    {
        UART_printf(
                    " REMOTE_SERVICE_MCAN2: Able to register remote service MCAN2 handler\n");
    }
    return status;
}

int32_t appRemoteServiceMcan2DeInit()
{
    int32_t status = 0;

    status = appRemoteServiceUnRegister(APP_REMOTE_SERVICE_MCAN2_NAME);
    if(status!=0)
    {
        UART_printf(
            " REMOTE_SERVICE_MCAN2: ERROR: Unable to register remote service MCAN2 handler\n");
    }

    return status;
}
static void data_sm(UArg arg0, UArg arg1)
{
    int i;
    while(1)
    {
        if(FLAG_SM == 0)
        {
            Task_sleep(333);
            FLAG_SM = 1;
        }
        else
        {
            Task_sleep(33);
        }
        uintptr_t addr = 0xbc7a0000;//jyo - 0xbcb70000;
        int obj = 0;
        appMemCacheInv((void*)addr, sizeof(st_OP_signals)*20);
        st_OP_signals *cmd_prms = (volatile st_OP_signals*)addr;
        printf("status : %d\n",cmd_prms[0].status);
        if(cmd_prms[0].status == STATUS_FILLED)
        {
           for (obj = 0; obj < 20; obj++)
           {
            //Copy all signals
            printf("long dist : %f\t",(cmd_prms[obj].long_distance));
            printf("lat dist : %f\t",(cmd_prms[obj].lat_distance));
            printf("ped_Speed_x : %f\t",(cmd_prms[obj].ped_Speed_y));
            printf("ped_Speed_y : %f\n",(cmd_prms[obj].ped_Speed_x));
           }
           cmd_prms[0].status = STATUS_EMPTY;
        }
        appMemCacheWbInv((void*)addr, sizeof(st_OP_signals)*20);
    }//while
}

int32_t appRemoteServiceMCAN2Handler(char *service_name, uint32_t cmd,
    void *prm, uint32_t prm_size, uint32_t flags)
{
    int32_t  status = CSL_PASS;
    SM_addr = (uintptr_t)(*(volatile uint32_t*)prm);
    printf("ADDRESS OF SHARED MEMORY : R5F = 0x%08x\n",SM_addr);
    return status;
}



#if defined(R5F) && defined(SYSBIOS)
int32_t appMCAN2DeInit()
{
    /* MCAN2_deInit(); */

   return 0;
}
#endif

#ifdef TIMER_CALL
#if (TX_CODE) || (TX_RX_CODE)
void Txtest()
{
   MCAN_ProtocolStatus protStatus;

   st_OP_Signal_1 pd_dcan_msg_final;
   st_OP_Signal_0 pd_dcan_msg;

   uint32_t obj = 0,loopCnt = 0,obj_count = 0;

   /* Enable Transmission interrupt */
   uint32_t status = MCAN_txBufTransIntrEnable(gMcanModAddr,
                                          1U,
                                          (uint32_t)TRUE);
   while(1)
   {
       Task_sleep(33);
#if 1
       if(SM_addr != NULL){
        appMemCacheInv((void*)SM_addr, sizeof(st_OP_signals)*20);
        st_OP_signals *cmd_prms = (volatile st_OP_signals*)SM_addr;
        printf("STATUS: %d\n",cmd_prms[0].status);
#endif
        if(cmd_prms[0].status == STATUS_FILLED)
        {
           for (obj = 0; obj < 5; obj++)
           {
               //Copy all signals
//               printf("long dist : %f\t",(cmd_prms[obj].long_distance));
//               printf("lat dist : %f\t",(cmd_prms[obj].lat_distance));
//               printf("ped_Speed_x : %f\t",(cmd_prms[obj].ped_Speed_y));
//               printf("ped_Speed_y : %f\n",(cmd_prms[obj].ped_Speed_x));

               txMsg.id  = (uint32_t)((uint32_t)(mcan2TxMsgId[obj]) << 18U);

               for (loopCnt = 0; loopCnt < 8; loopCnt++)
               {
                   pd_dcan_msg_final.pd_msg[loopCnt] = 0;
               }
                pd_dcan_msg.dist_x = (cmd_prms[obj].long_distance);
                pd_dcan_msg.dist_y = (cmd_prms[obj].lat_distance);
                pd_dcan_msg.vel_y = (cmd_prms[obj].ped_Speed_y);
                pd_dcan_msg.vel_x = (cmd_prms[obj].ped_Speed_x);
//               pd_dcan_msg.dist_x = 1;
//               pd_dcan_msg.dist_y = 11;
//               pd_dcan_msg.vel_y = 21;
//               pd_dcan_msg.vel_x = 31;

                pd_dcan_msg.obj_type = 5;
                pd_dcan_msg.roll_count = rolling_count;

                //Convert all signals to be signed
                pd_dcan_msg_final.st_PD_Fields.obj_type = pd_dcan_msg.obj_type;
                pd_dcan_msg_final.st_PD_Fields.roll_count = pd_dcan_msg.roll_count;

                pd_dcan_msg_final.st_PD_Fields.dist_x = (pd_dcan_msg.dist_x + 8192);
                pd_dcan_msg_final.st_PD_Fields.vel_x = (pd_dcan_msg.vel_x + 4096);
                pd_dcan_msg_final.st_PD_Fields.dist_y = (pd_dcan_msg.dist_y + 8192);
                pd_dcan_msg_final.st_PD_Fields.vel_y = (pd_dcan_msg.vel_y + 4096);

                for (loopCnt = 0; loopCnt < 8; loopCnt++)
                {
                    txMsg.data[loopCnt] = pd_dcan_msg_final.pd_msg[loopCnt];
                }


               if (CSL_PASS != status)
               {
                   UART_printf("\nError in enabling buffer Transmit interrupt...\n");
               }
               else
               {
                   txMsg.dlc = 8;
                   /* Write message to Msg RAM */
                   MCAN_writeMsgRam(gMcanModAddr,
                                    MCAN_MEM_TYPE_BUF,
                                    1U,
                                    &txMsg);
                   /* Add request for transmission */
                   status = MCAN_txBufAddReq(gMcanModAddr, 1U);
                   if (CSL_PASS != status)
                   {
                       UART_printf("\nError in Adding Transmission Request...\n");
                       break;
                   }
                   //printf("in semaphore pend\n");
                   SemaphoreP_pend(TxgSyncSem, SemaphoreP_WAIT_FOREVER);
                   //printf("out semaphore pend\n");
                   MCAN_getProtocolStatus(gMcanModAddr, &protStatus);

                   /* Checking for Errors */
                   if (((MCAN_ERR_CODE_NO_ERROR == protStatus.lastErrCode) ||
                        (MCAN_ERR_CODE_NO_CHANGE == protStatus.lastErrCode)) &&
                       ((MCAN_ERR_CODE_NO_ERROR == protStatus.dlec) ||
                        (MCAN_ERR_CODE_NO_CHANGE == protStatus.dlec)) &&
                       (0U == protStatus.pxe))
                   {
                    //UART_printf("\nMessage successfully transferred with payload Bytes \n");
                       num_Tx++;
                   }
                   else
                   {
                       UART_printf("\nError in transmission with payload Bytes:%d\n");
                       status = CSL_EFAIL;
                       break;
                   }
               }
           }// for
#if 1
           cmd_prms[0].status = STATUS_EMPTY;
           if(rolling_count==63)
           {
               rolling_count=0;
           }
           else
           {
               rolling_count++;
           }
#endif
        }//if
        appMemCacheWbInv((void*)SM_addr, sizeof(st_OP_signals)*20);}
   }//while
}

#if defined(R5F) && defined(SYSBIOS)
static Void TxMain(UArg arg0, UArg arg1)
{
    UART_printf("Tx thread\n");
    Txtest();
}
static Void RxMain(UArg arg0, UArg arg1)
{
    UART_printf("Rx thread\n");
    Rxtest();
}
void Tx_Timer_task()
{

    printf("TASK SLEEP APPROACH \n");
    uint32_t timePeriodInMilliSec = 33;  //3 sec
    SemaphoreP_Params semParams;

    SemaphoreP_Params_init(&semParams);
    semParams.mode = SemaphoreP_Mode_COUNTING;
    TxgSyncSem = SemaphoreP_create(0U, &semParams);

    Task_Params tskParams1;
   Task_Params_init(&tskParams1);

   tskParams1.arg0 = (UArg) NULL;
   tskParams1.arg1 = (UArg) NULL;
   tskParams1.priority = 8u;
   tskParams1.stack = gTskTxStackMain;
   tskParams1.stackSize = sizeof (gTskTxStackMain);
   Task_create(TxMain, &tskParams1, NULL);

   Task_Params tskParams2;
   Task_Params_init(&tskParams2);

   tskParams2.arg0 = (UArg) NULL;
   tskParams2.arg1 = (UArg) NULL;
   tskParams2.priority = 8u;
   tskParams2.stack = gTskRxStackMain;
   tskParams2.stackSize = sizeof (gTskRxStackMain);
   Task_create(RxMain, &tskParams2, NULL);
   //Task_create(data_sm, &tskParams2, NULL);
}
#endif
#endif

#if (RX_CODE) || (TX_RX_CODE)
#if defined(R5F) && defined(SYSBIOS)
void Rxtest()
{
    int32_t              testStatus = CSL_PASS;
    MCAN_ErrCntStatus    errCounter;
    MCAN_ProtocolStatus protStatus;
    MCAN_RxFIFOStatus fifoStatus;
    while(1)
    {
        SemaphoreP_pend(RxgSyncSem, SemaphoreP_WAIT_FOREVER);

        if(((gMcanIsrIntr1Status & MCAN_INTR_SRC_RX_FIFO0_NEW_MSG) ==
                                    MCAN_INTR_SRC_RX_FIFO0_NEW_MSG))
        {
            fifoStatus.num = (uint32_t)MCAN_RX_FIFO_NUM_0;
        }
        else
        {
            fifoStatus.num = (uint32_t)MCAN_RX_FIFO_NUM_1;
        }

        MCAN_getRxFIFOStatus(APP_MCAN2_BASEADDR, &fifoStatus);
        MCAN_readMsgRam(APP_MCAN2_BASEADDR,
                        MCAN_MEM_TYPE_FIFO,
                        fifoStatus.getIdx,
                        (uint32_t)fifoStatus.num,
                        &rxMsg);
        (void) MCAN_writeRxFIFOAck(APP_MCAN2_BASEADDR,
                                   (uint32_t)fifoStatus.num,
                                   fifoStatus.getIdx);

        MCAN_getProtocolStatus(APP_MCAN2_BASEADDR, &protStatus);

       /* Checking for Errors */
       if (((MCAN_ERR_CODE_NO_ERROR == protStatus.lastErrCode) ||
            (MCAN_ERR_CODE_NO_CHANGE == protStatus.lastErrCode)) &&
           ((MCAN_ERR_CODE_NO_ERROR == protStatus.dlec) ||
            (MCAN_ERR_CODE_NO_CHANGE == protStatus.dlec)) &&
           (0U == protStatus.pxe))
       {
//           UART_printf("\n(%dth) Message successfully received.%d , %d, %d ,%d \n",
//                       rxMsg.id,rxMsg.data[0],rxMsg.data[1],rxMsg.data[2],rxMsg.data[3]);
           num_Rx++;
       }
       else
       {
           UART_printf("\nError in reception message.\n");
       }
    }//while
}
void Rx_Timer_task()
{
    int32_t status = CSL_PASS;

    Board_pinmuxUpdate(gCockpitDemoPinmuxDataInfo,
                       BOARD_SOC_DOMAIN_MAIN);

    /* Set drive strength */
    CSL_REG32_WR(CSL_WKUP_CTRL_MMR0_CFG0_BASE +
            CSL_WKUP_CTRL_MMR_CFG0_H_IO_DRVSTRNGTH0_PROXY, 0xFU);

    CSL_REG32_WR(CSL_WKUP_CTRL_MMR0_CFG0_BASE +
            CSL_WKUP_CTRL_MMR_CFG0_V_IO_DRVSTRNGTH0_PROXY, 0xFU);

    SET_DEVICE_STATE(TISCI_DEV_MCAN2,
            TISCI_MSG_VALUE_DEVICE_SW_STATE_ON,
            status);

    SemaphoreP_Params semParams;

    SemaphoreP_Params_init(&semParams);
    semParams.mode = SemaphoreP_Mode_COUNTING;
    RxgSyncSem = SemaphoreP_create(0U, &semParams);
}
#endif

#endif
#endif
